@@ -69,13 +69,12 @@ struct _Slide2AnswerStatusPluginPrivate
   GtkWidget* call_window_number_label;
   GtkWidget* call_window_slider;
   gboolean is_voip_call;
-  DBusGConnection *dbus_conn;
   DBusGConnection *dbus_system_conn;
-  DBusGProxy *dbus_proxy;
   DBusGProxy *dbus_system_proxy;
   OssoABookAggregator* abook_aggregator;
   OssoABookAccountManager* abook_account_manager;
   TpChannel* active_channel;
+  GHashTable* account_map;
 };
 
 HD_DEFINE_PLUGIN_MODULE(Slide2AnswerStatusPlugin, slide2answer_status_plugin, HD_TYPE_STATUS_MENU_ITEM);
@@ -167,43 +166,53 @@ _slider_changed(GtkRange* range,
 		gdouble value,
 		gpointer user_data)
 {
-  g_warning("slider changed\n");
-  g_warning("scroll value %f\n", value);
-  Slide2AnswerStatusPlugin* plugin = SLIDE2ANSWER_STATUS_PLUGIN(user_data);
-  if(value>0.9)
+  if(IS_SLIDE2ANSWER_STATUS_PLUGIN(user_data))
   {
-    gtk_widget_hide(GTK_WIDGET(plugin->priv->call_window));
-    _accept_call(plugin);
+    Slide2AnswerStatusPlugin* plugin = SLIDE2ANSWER_STATUS_PLUGIN(user_data);
+    if(value>0.9)
+    {
+      gtk_widget_hide(GTK_WIDGET(plugin->priv->call_window));
+      _accept_call(plugin);
+    }
+    else if(value<-0.9)
+    {
+      gtk_widget_hide(GTK_WIDGET(plugin->priv->call_window));
+      _reject_call(plugin);
+    }
   }
-  else if(value<-0.9)
+  return FALSE;
+}
+
+static gboolean
+_slider_changed_stopped(GtkWidget *widget,
+			GdkEventButton *event,
+			gpointer user_data)
+{
+  if(GTK_IS_RANGE(user_data))
   {
-    gtk_widget_hide(GTK_WIDGET(plugin->priv->call_window));
-    _reject_call(plugin);
+    gtk_range_set_value(GTK_RANGE(user_data), 0.0);
   }
   return FALSE;
 }
 
-static void
-_slider_changed_stopped(GtkWidget *widget,
-			GdkEventButton *event,
-			gpointer user_data)
-{
-  g_warning("stopped\n");
-  gtk_range_set_value(GTK_RANGE(user_data), 0.0);
-}
-
 static void
 _create_slide2answer_window(Slide2AnswerStatusPlugin* status_plugin)
 {
-  g_warning("create slide 2 answer window\n");
-  status_plugin->priv->call_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-   
+  status_plugin->priv->call_window = hildon_window_new();//gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  hildon_gtk_window_set_portrait_flags(GTK_WINDOW(status_plugin->priv->call_window),
+				       HILDON_PORTRAIT_MODE_SUPPORT);
+  /*
   GdkPixbuf* pixbuf = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
 					       "general_default_avatar",
 					       AVATAR_SIZE, GTK_ICON_LOOKUP_NO_SVG, NULL);
-  status_plugin->priv->call_window_avatar = gtk_image_new_from_pixbuf(pixbuf);
+  */
+  //  status_plugin->priv->call_window_avatar = gtk_image_new_from_pixbuf(pixbuf);
+  
+  status_plugin->priv->call_window_avatar = osso_abook_avatar_image_new();
 
-  g_object_unref(pixbuf);
+  osso_abook_avatar_image_set_fallback_icon(OSSO_ABOOK_AVATAR_IMAGE(status_plugin->priv->call_window_avatar),
+				      "general_default_avatar");
+  //  g_object_unref(pixbuf);
   status_plugin->priv->call_window_name_label = gtk_label_new("");
   hildon_helper_set_logical_font(status_plugin->priv->call_window_name_label, "LargeSystemFont");
   status_plugin->priv->call_window_number_label = gtk_label_new("");
@@ -245,13 +254,11 @@ _create_slide2answer_window(Slide2AnswerStatusPlugin* status_plugin)
   gtk_box_pack_start(GTK_BOX(contact_info_box), status_plugin->priv->call_window_name_label, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(contact_info_box), status_plugin->priv->call_window_number_label, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(contact_box), contact_info_box, TRUE, TRUE, 0);
-  
   gtk_box_pack_start(GTK_BOX(button_box), reject_button, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(button_box), accept_button, TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(content), contact_box, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(content), contact_box, TRUE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(content), status_plugin->priv->call_window_slider, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(content), button_box, TRUE, FALSE, 0);
-  g_warning("content show all\n");
   gtk_widget_show_all(content);
 }
 
@@ -261,9 +268,7 @@ _show_slide2answer_window(Slide2AnswerStatusPlugin* plugin)
   if(plugin->priv->call_window == NULL)
   {
     _create_slide2answer_window(plugin);
-    g_warning("show window\n");
-    gtk_widget_show(GTK_WIDGET(plugin->priv->call_window));
-    g_warning("show fullscreen\n");
+    gtk_widget_show_all(GTK_WIDGET(plugin->priv->call_window));
     gtk_window_fullscreen(GTK_WINDOW(plugin->priv->call_window));
     Display* xdisplay = GDK_WINDOW_XDISPLAY(plugin->priv->call_window->window);
     guint32 enable = {9};
@@ -277,7 +282,7 @@ _show_slide2answer_window(Slide2AnswerStatusPlugin* plugin)
   else
   {
     gtk_range_set_value(GTK_RANGE(plugin->priv->call_window_slider), 0.0);
-    gtk_widget_show(GTK_WIDGET(plugin->priv->call_window));
+    gtk_widget_show_all(GTK_WIDGET(plugin->priv->call_window));
   }
 }
 
@@ -354,13 +359,15 @@ _set_unknowncaller_data(Slide2AnswerStatusPlugin* status_plugin, const gchar* id
 		             identifier);
   gtk_label_set_text(GTK_LABEL(status_plugin->priv->call_window_number_label),
 		             "");
-  GdkPixbuf* pixbuf = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-                                               "general_default_avatar",
-                                               AVATAR_SIZE, GTK_ICON_LOOKUP_NO_SVG, NULL);
+  osso_abook_avatar_image_set_avatar(OSSO_ABOOK_AVATAR_IMAGE(status_plugin->priv->call_window_avatar),
+				     NULL);
+  /* GdkPixbuf* pixbuf = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), */
+  /* 					       "general_default_avatar", */
+  /* 					       216, GTK_ICON_LOOKUP_NO_SVG, NULL); */
+  /* osso_abook_avatar_image_set_pixbuf(OSSO_ABOOK_AVATAR_IMAGE(status_plugin->priv->call_window_avatar), */
+  /* 				     pixbuf); */
+  /* g_object_unref(pixbuf); */
 
-  gtk_image_set_from_pixbuf(GTK_IMAGE(status_plugin->priv->call_window_avatar),
-			                pixbuf);
-  g_object_unref(pixbuf);
 }
 
 static void
@@ -370,73 +377,8 @@ _set_contact_data(Slide2AnswerStatusPlugin* status_plugin, OssoABookContact* con
 		     osso_abook_contact_get_display_name(contact));
   gtk_label_set_text(GTK_LABEL(status_plugin->priv->call_window_number_label),
   		     identifier);      
-  GdkPixbuf* pixbuf = osso_abook_avatar_get_image_scaled(OSSO_ABOOK_AVATAR(contact),
-						AVATAR_SIZE,
-						AVATAR_SIZE,
-						FALSE);
-             
-  /*GdkPixbuf* pixbuf = osso_abook_contact_get_photo(contact);
-  if(!pixbuf)
-  {
-    pixbuf = osso_abook_avatar_get_image_scaled(OSSO_ABOOK_AVATAR(contact),
-						AVATAR_SIZE,
-						AVATAR_SIZE,
-						FALSE);
-  }*/
-  if(!pixbuf)
-  {
-    gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-			     "general_default_avatar",
-			     AVATAR_SIZE, GTK_ICON_LOOKUP_NO_SVG, NULL);
-  }
-
-  gtk_image_set_from_pixbuf(GTK_IMAGE(status_plugin->priv->call_window_avatar),
-			    pixbuf);
-  g_object_unref(pixbuf);
-}
-
-static void
-_list_roster_contacts(Slide2AnswerStatusPlugin* status_plugin)
-{
-  GList* roster_contacts = osso_abook_aggregator_list_roster_contacts(status_plugin->priv->abook_aggregator);
-  GList* roster;
-  for(roster = roster_contacts; roster; roster = roster->next)
-  {
-    OssoABookContact* roster_contact = (OssoABookContact*) roster->data;
-    gchar* name = g_strdup(osso_abook_contact_get_persistent_uid(roster_contact));
-    g_warning("list roster %s\n", name);
-    g_free(name);
-  }
-  g_list_free(roster_contacts);
-}
-
-static void
-_account_changed(OssoABookAccountManager *manager,
-		 McAccount *account,
-		 guint property,
-		 GValue *value,
-		 gpointer user_data)
-{
-  g_warning("slide2answer account changed\n");
-  g_warning("name %s\n", mc_account_get_connection_path(account));
-}
-
-static void
-_account_created(OssoABookAccountManager *manager,
-		 McAccount *account,
-		 gpointer user_data)
-{
-  g_warning("slide2answer account created\n");
-  g_warning("name %s\n", mc_account_get_connection_path(account));
-}
-
-static void
-_account_removed(OssoABookAccountManager *manager,
-		 McAccount *account,
-		 gpointer user_data)
-{
-  g_warning("slide2answer account removed\n");
-  g_warning("name %s\n", mc_account_get_connection_path(account));
+  osso_abook_avatar_image_set_avatar(OSSO_ABOOK_AVATAR_IMAGE(status_plugin->priv->call_window_avatar),
+				     OSSO_ABOOK_AVATAR(contact));
 }
 
 static void
@@ -449,15 +391,10 @@ handle_new_channel(TpConnection *proxy,
 		   gpointer user_data,
 		   GObject *weak_object)
 {
-  g_warning("new channel path %s\n", arg_Object_Path);
-  g_warning("new channel type %s\n", arg_Channel_Type);
-  g_warning("suppress handler %d\n", arg_Suppress_Handler);
-  g_warning("handle type %d \n", arg_Handle_Type);
   if(arg_Suppress_Handler || 
      g_strcmp0(arg_Channel_Type, "org.freedesktop.Telepathy.Channel.Type.Text") == 0)
     return;
   
-  
   Slide2AnswerStatusPlugin* plugin = SLIDE2ANSWER_STATUS_PLUGIN(user_data);
   plugin->priv->is_voip_call = !g_str_has_prefix(arg_Object_Path, "/org/freedesktop/Telepathy/Connection/ring/tel/ring");
   TpChannel* channel = tp_channel_new (proxy, 
@@ -479,7 +416,6 @@ handle_new_channel(TpConnection *proxy,
       contacts = osso_abook_aggregator_find_contacts_for_phone_number(plugin->priv->abook_aggregator,
 								      identifier,
 								      TRUE);
-
     if(contacts)
     {
       GList* contact = contacts;
@@ -490,62 +426,112 @@ handle_new_channel(TpConnection *proxy,
 	while(contact)
 	{
 	  OssoABookContact* abook_contact = OSSO_ABOOK_CONTACT(contact->data);
-	  g_warning("display name %s\n", osso_abook_contact_get_display_name(abook_contact));
 	  contact = contact->next;
 	}
       }
       g_list_free(contacts);
-    }else{
-        _set_unknowncaller_data(plugin, identifier);
     }
-  }else{
+    else
+    {
+      _set_unknowncaller_data(plugin, identifier);
+    }
+  }
+  else
+  {
     _set_unknowncaller_data(plugin, dgettext("rtcom-call-ui", "voip_fi_caller_information_unknown_caller"));
   }
-
-  g_warning("identifier >%s<\n", identifier);
   plugin->priv->active_channel = channel;
   //  g_object_unref(channel);
 }
 
+static void
+_account_register_new_channel(Slide2AnswerStatusPlugin* status_plugin, McAccount* account)
+{
+  if(mc_account_get_connection_path(account)!=NULL)
+  {
+    GError* error = NULL;
+    TpDBusDaemon* tpdbus = tp_dbus_daemon_dup(NULL);
+    TpConnection* tpconnection = tp_connection_new(tpdbus, 
+						   NULL, 
+						   mc_account_get_connection_path(account), 
+						   &error);
+    if(error)
+    {
+      g_warning("error creating connection %s\n", error->message);
+      g_error_free(error);
+      error = NULL;
+    }
+    else
+    {
+      tp_cli_connection_connect_to_new_channel(tpconnection,
+					       handle_new_channel,
+					       status_plugin,
+					       NULL,
+					       NULL, 
+					       NULL);
+    }
+  }
+}
+
+static void
+_account_changed(OssoABookAccountManager *manager,
+		 McAccount *account,
+		 guint property,
+		 GValue *value,
+		 gpointer user_data)
+{
+  Slide2AnswerStatusPlugin* status_plugin = SLIDE2ANSWER_STATUS_PLUGIN(user_data);
+  TpConnectionStatus stat = mc_account_get_connection_status(account);
+  const gchar* connection = mc_account_get_connection_path(account);
+  if((connection != NULL) && (stat == TP_CONNECTION_STATUS_CONNECTED))
+  {
+    if(g_hash_table_lookup(status_plugin->priv->account_map, connection) == NULL)
+    {
+      g_hash_table_insert(status_plugin->priv->account_map, g_strdup(connection), g_strdup(connection));
+      _account_register_new_channel(status_plugin, account);
+    }
+  }
+  else if((connection != NULL) && (stat == TP_CONNECTION_STATUS_DISCONNECTED))
+  {
+    if(g_hash_table_lookup(status_plugin->priv->account_map, connection) != NULL)
+    {
+      g_hash_table_remove(status_plugin->priv->account_map, connection);
+    }
+  }
+}
+
+static void
+_account_created(OssoABookAccountManager *manager,
+		 McAccount *account,
+		 gpointer user_data)
+{
+  g_warning("slide2answer account created\n");
+  g_warning("name %s\n", mc_account_get_connection_path(account));
+}
+
+static void
+_account_removed(OssoABookAccountManager *manager,
+		 McAccount *account,
+		 gpointer user_data)
+{
+  g_warning("slide2answer account removed\n");
+  g_warning("name %s\n", mc_account_get_connection_path(account));
+}
+
 static void
 account_ready_cb(OssoABookAccountManager *manager,
 		 const GError *xerror,
 		 gpointer data)
 {
   GList* accounts = osso_abook_account_manager_list_accounts(manager,
-							      NULL, NULL);
+							     NULL, NULL);
   if(accounts)
   {
     GList* account;
     for(account = accounts; account; account = account->next)
     {
       McAccount* mc_account = MC_ACCOUNT(account->data);
-      g_warning("account %s\n", mc_account_get_display_name(mc_account));
-      g_warning("path %s \n", mc_account_get_connection_path(mc_account));
-      if(mc_account_get_connection_path(mc_account)!=NULL)
-      {
-	GError* error = NULL;
-	TpDBusDaemon* tpdbus = tp_dbus_daemon_dup(NULL);
-	TpConnection* tpconnection = tp_connection_new(tpdbus, 
-						       NULL, 
-						       mc_account_get_connection_path(mc_account), 
-						       &error);
-	if(error)
-	{
-	  g_warning("error creating connection %s\n", error->message);
-	  g_error_free(error);
-	  error = NULL;
-	}
-	else
-	{
-	  tp_cli_connection_connect_to_new_channel(tpconnection,
-						   handle_new_channel,
-						   data,
-						   NULL,
-						   NULL, 
-						   NULL);
-	}
-      }
+      _account_register_new_channel(SLIDE2ANSWER_STATUS_PLUGIN(data), mc_account);
     }
     g_list_free(accounts);
   }
@@ -559,16 +545,6 @@ account_ready_cb(OssoABookAccountManager *manager,
 		   G_CALLBACK(_account_removed), data);
 }
 
-static void
-aggregator_ready_cb(OssoABookWaitable *waitable,
-		    const GError *error,
-		    gpointer data)
-{
-  Slide2AnswerStatusPlugin* plugin = SLIDE2ANSWER_STATUS_PLUGIN(data);
-  _list_roster_contacts(plugin);
-}
-
-
 static void
 slide2answer_status_plugin_init(Slide2AnswerStatusPlugin* status_plugin)
 {
@@ -579,6 +555,10 @@ slide2answer_status_plugin_init(Slide2AnswerStatusPlugin* status_plugin)
   status_plugin->priv->is_voip_call = FALSE;
   status_plugin->priv->abook_account_manager = osso_abook_account_manager_new();
   status_plugin->priv->active_channel = NULL;
+  status_plugin->priv->account_map = g_hash_table_new_full(g_str_hash, 
+							   g_str_equal, 
+							   g_free, 
+							   g_free);
 
   EBookQuery* ebook_query = e_book_query_from_string("(not (is_vcard \"X-TELEPATHY-BLOCKED\"  \"yes\"))");
   osso_abook_account_manager_set_roster_query(status_plugin->priv->abook_account_manager, ebook_query);
@@ -594,9 +574,6 @@ slide2answer_status_plugin_init(Slide2AnswerStatusPlugin* status_plugin)
   osso_abook_roster_start(OSSO_ABOOK_ROSTER(status_plugin->priv->abook_aggregator));
   osso_abook_account_manager_call_when_ready(status_plugin->priv->abook_account_manager,
 					     account_ready_cb, status_plugin, NULL);
-  osso_abook_waitable_call_when_ready(OSSO_ABOOK_WAITABLE(status_plugin->priv->abook_aggregator),
-				      aggregator_ready_cb, status_plugin, NULL);
-  
   g_signal_connect(status_plugin->priv->call_window, "delete-event", G_CALLBACK(&gtk_widget_hide_on_delete), NULL);
   register_incoming_call_handler(status_plugin);
 }
@@ -614,7 +591,8 @@ slide2answer_status_plugin_finalize(GObject* object)
   {
     g_object_unref(status_plugin->priv->abook_aggregator);
   }
-  dbus_g_connection_unref(status_plugin->priv->dbus_conn);
+  g_hash_table_remove_all(status_plugin->priv->account_map);
+  g_object_unref(status_plugin->priv->account_map);
   G_OBJECT_CLASS(slide2answer_status_plugin_parent_class)->finalize (object);
 }
 
@@ -629,4 +607,3 @@ slide2answer_status_plugin_class_init(Slide2AnswerStatusPluginClass* klass)
   g_type_class_add_private(klass, sizeof(Slide2AnswerStatusPluginPrivate));
   G_OBJECT_CLASS(klass)->finalize = (GObjectFinalizeFunc)slide2answer_status_plugin_finalize;
 }
-
